"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var PrefixExpression = /** @class */ (function () {
    function PrefixExpression(Token, // The prefix token, e.g. !
    Operator, Right, NodeName) {
        if (NodeName === void 0) { NodeName = "PrefixExpression"; }
        this.Token = Token;
        this.Operator = Operator;
        this.Right = Right;
        this.NodeName = NodeName;
    }
    PrefixExpression.prototype.expressionNode = function () { };
    PrefixExpression.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    PrefixExpression.prototype.String = function () {
        var out = "(";
        out += this.Operator;
        out += this.Right.String();
        out += ")";
        return out;
    };
    PrefixExpression.prototype.UnParse = function (ctx) {
        return "(" + this.Operator + ctx.transpile(this.Right) + ")";
    };
    return PrefixExpression;
}());
exports.PrefixExpression = PrefixExpression;
var InfixExpression = /** @class */ (function () {
    function InfixExpression(Token, // The operator token, e.g. +
    Left, Operator, Right, NodeName) {
        if (NodeName === void 0) { NodeName = "InfixExpression"; }
        this.Token = Token;
        this.Left = Left;
        this.Operator = Operator;
        this.Right = Right;
        this.NodeName = NodeName;
    }
    InfixExpression.prototype.expressionNode = function () { };
    InfixExpression.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    InfixExpression.prototype.String = function () {
        var out = "(";
        out += this.Left.String();
        out += " " + this.Operator + " ";
        out += (this.Right.String());
        out += ")";
        return out;
    };
    InfixExpression.prototype.UnParse = function (ctx) {
        return "(" + ctx.transpile(this.Left) + " " +
            this.Operator + " " + ctx.transpile(this.Right) + ")";
    };
    return InfixExpression;
}());
exports.InfixExpression = InfixExpression;
var CallExpression = /** @class */ (function () {
    function CallExpression(Token, // The '(' token
    Function, // Identifier or FunctionLiteral
    Arguments, NodeName) {
        if (Arguments === void 0) { Arguments = []; }
        if (NodeName === void 0) { NodeName = "CallExpression"; }
        this.Token = Token;
        this.Function = Function;
        this.Arguments = Arguments;
        this.NodeName = NodeName;
    }
    CallExpression.prototype.expressionNode = function () { };
    CallExpression.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    CallExpression.prototype.String = function () {
        var out = "", args = [];
        this.Arguments.forEach(function (a) {
            args.push(a.String());
        });
        out += this.Function.String();
        out += "(";
        out += args.join(", ");
        out += ")";
        return out;
    };
    CallExpression.prototype.UnParse = function (ctx) {
        var callArgs = this.Arguments.map(function (arg) { return ctx.transpile(arg); });
        return ctx.transpile(this.Function) + "(" + callArgs.join(",") + ")";
    };
    return CallExpression;
}());
exports.CallExpression = CallExpression;
var NewExpression = /** @class */ (function () {
    function NewExpression(Token, // the token.NEW token
    Name, NodeName) {
        if (NodeName === void 0) { NodeName = "NewExpression"; }
        this.Token = Token;
        this.Name = Name;
        this.NodeName = NodeName;
    }
    NewExpression.prototype.expressionNode = function () { };
    NewExpression.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    NewExpression.prototype.String = function () {
        var out = this.TokenLiteral() + " " + out + this.Name.String();
        return out;
    };
    NewExpression.prototype.UnParse = function (ctx) {
        return "new " + this.Name.Value;
    };
    return NewExpression;
}());
exports.NewExpression = NewExpression;
var ExecExpression = /** @class */ (function () {
    function ExecExpression(Token, // the token.NEW token
    Name, Callback, NodeName) {
        if (NodeName === void 0) { NodeName = "ExecExpression"; }
        this.Token = Token;
        this.Name = Name;
        this.Callback = Callback;
        this.NodeName = NodeName;
    }
    ExecExpression.prototype.expressionNode = function () { };
    ExecExpression.prototype.TokenLiteral = function () {
        return this.Token.Literal;
    };
    ExecExpression.prototype.String = function () {
        var out = this.TokenLiteral() + " " + out + this.Name.String();
        return out;
    };
    ExecExpression.prototype.UnParse = function (ctx) {
        var out = "";
        return out;
    };
    return ExecExpression;
}());
exports.ExecExpression = ExecExpression;
var StreamExpression = /** @class */ (function () {
    function StreamExpression(Token, // The '->' or '<- tokens
    Direction, Source, Sink, Transforms, NodeName) {
        if (Transforms === void 0) { Transforms = []; }
        if (NodeName === void 0) { NodeName = "StreamExpression"; }
        this.Token = Token;
        this.Direction = Direction;
        this.Source = Source;
        this.Sink = Sink;
        this.Transforms = Transforms;
        this.NodeName = NodeName;
    }
    StreamExpression.prototype.expressionNode = function () { };
    StreamExpression.prototype.TokenLiteral = function () { return this.Token.Literal; };
    StreamExpression.prototype.String = function () {
        var out = "";
        out += this.Source.String() + " " + this.Transforms
            .map(function (t) { return t.String(); })
            .join(" " + this.TokenLiteral() + " ")
            + this.Sink.String();
        return out;
    };
    StreamExpression.prototype.UnParse = function (ctx) {
        return ctx.unParse(this);
    };
    return StreamExpression;
}());
exports.StreamExpression = StreamExpression;
var IndexExpression = /** @class */ (function () {
    function IndexExpression(Token, // The [ token
    Left, Index, NodeName) {
        if (NodeName === void 0) { NodeName = "IndexExpression"; }
        this.Token = Token;
        this.Left = Left;
        this.Index = Index;
        this.NodeName = NodeName;
    }
    IndexExpression.prototype.expressionNode = function () { };
    IndexExpression.prototype.TokenLiteral = function () { return this.Token.Literal; };
    IndexExpression.prototype.String = function () {
        var out = "";
        out += this.Left.String();
        out += ".";
        out += this.Index.String();
        return out;
    };
    IndexExpression.prototype.UnParse = function (ctx) {
        var transpiledIndex = ctx.transpile(this.Index);
        return ctx.transpile(this.Left) + "[" + transpiledIndex + "]";
    };
    return IndexExpression;
}());
exports.IndexExpression = IndexExpression;
var IndexAssignmentExpression = /** @class */ (function () {
    function IndexAssignmentExpression(Token, // The [ token
    Left, Index, Assignment, NodeName) {
        if (NodeName === void 0) { NodeName = "IndexAssignmentExpression"; }
        this.Token = Token;
        this.Left = Left;
        this.Index = Index;
        this.Assignment = Assignment;
        this.NodeName = NodeName;
    }
    IndexAssignmentExpression.prototype.expressionNode = function () { };
    IndexAssignmentExpression.prototype.TokenLiteral = function () { return this.Token.Literal; };
    IndexAssignmentExpression.prototype.String = function () {
        var out = "";
        out += this.Left.String();
        out += ".";
        out += this.Index.String();
        out += "=";
        out += this.Assignment.String();
        return out;
    };
    IndexAssignmentExpression.prototype.UnParse = function (ctx) {
        return ctx.transpile(this.Left) + "[" + ctx.transpile(this.Index) + "] = " + ctx.transpile(this.Assignment) + ";";
    };
    return IndexAssignmentExpression;
}());
exports.IndexAssignmentExpression = IndexAssignmentExpression;
