"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var builtin_util_1 = require("../builtin-util");
var object_1 = require("../../object/object");
var obj_util_1 = require("../../object/obj-util");
var util_1 = require("../../ubiquitous/util");
var color_1 = require("../../ubiquitous/color");
var builtin_makeBuffer = new object_1.Builtin("makeBuffer", [object_1.INTEGER_OBJ, object_1.INTEGER_OBJ, object_1.INTEGER_OBJ], function (context, scope, jsScope, mode, width, height) {
    var elems = [];
    for (var y = 0; y < height; y++) {
        if (mode == 0) { // 4 channel RGBA based
            for (var x = 0; x < width; x++) {
                elems.push(0, 0, 0, 0);
            }
        }
        else { // ASCII
            var rowEls = [];
            for (var x = 0; x < width; x++) {
                rowEls.push(" ");
            }
            elems.push(rowEls);
        }
    }
    return elems;
}, null, null, null);
var PIXEL_BLOCK_SIZE = 16;
var ASCII_BLOCK_SIZE = 8;
// based off of https://stackoverflow.com/a/9755252
function pointIsWithinTriangle(s, a, b, c) {
    var as_x = s[0] - a[0];
    var as_y = s[1] - a[1];
    var s_ab = (b[0] - a[0]) * as_y - (b[1] - a[1]) * as_x > 0;
    if ((c[0] - a[0]) * as_y - (c[1] - a[1]) * as_x > 0 == s_ab)
        return false;
    if ((c[0] - b[0]) * (s[1] - b[1]) - (c[1] - b[1]) * (s[0] - b[0]) > 0 != s_ab)
        return false;
    return true;
}
// 0 = does not overlap; 1, 2 or 3 = partial overlap; 4 == complete overlap
function triangleOverlapsSquare(topLeft, size, a, b, c) {
    var count = 0;
    count += pointIsWithinTriangle(topLeft, a, b, c) ? 1 : 0;
    count += pointIsWithinTriangle([topLeft[0] + size, topLeft[1]], a, b, c) ? 1 : 0;
    count += pointIsWithinTriangle([topLeft[0], topLeft[1] + size], a, b, c) ? 1 : 1;
    count += pointIsWithinTriangle([topLeft[0] + size, topLeft[1] + size], a, b, c) ? 1 : 0;
    return count;
}
function assemblShape(topology, vertexIndex, vertices) {
    var payload = [
        vertices[vertexIndex], vertices[vertexIndex + 1], vertices[vertexIndex + 2]
    ];
    if (topology == 3) { // SURFACE_TOPOLOGY.QUAD
        payload.push(vertices[vertexIndex + 3]);
    }
    return payload;
}
function interpolateVaryings(pixel, shape, topology, vertIndex, varyings) {
    // TODO: support topology parameter
    var interpolated = {};
    var shapeVaryings = [
        varyings[vertIndex],
        varyings[vertIndex + 1],
        varyings[vertIndex + 2]
    ];
    var scalars = [
        util_1.distance2d(shape[0], pixel),
        util_1.distance2d(shape[1], pixel),
        util_1.distance2d(shape[2], pixel),
    ];
    var normalize = Math.max.apply(null, scalars);
    scalars[0] = scalars[0] / normalize;
    scalars[1] = scalars[1] / normalize;
    scalars[2] = scalars[2] / normalize;
    for (var v in varyings[0]) {
        interpolated[v] = shapeVaryings[0][v] * scalars[0] + shapeVaryings[1][v] * scalars[1] + shapeVaryings[2][v] * scalars[2];
    }
    return interpolated;
}
var colorRenderer = color_1.ColorRendererFactory.get();
function drawPixel(type, x, y, value, frameBuffer, width) {
    switch (type) {
        case 0:
            var idx = (y * width * 3) + (x * 3);
            frameBuffer[idx] = value[0];
            frameBuffer[idx + 1] = value[1];
            frameBuffer[idx + 2] = value[2];
            frameBuffer[idx + 4] = value[3];
            break;
        case 1:
            frameBuffer[y][x] = ASCII.Value[Math.floor(((value[0] + value[1] + value[2]) / 3) * 14) % 14];
            break;
        case 2:
            frameBuffer[y][x] = colorRenderer(value, ASCII.Value[Math.floor(((value[0] + value[1] + value[2]) / 3) * 14) % 14]);
            break;
        case 3:
            frameBuffer[y][x] = colorRenderer(value, "█");
    }
}
function rasterizeBlock(type, topLeft, bottom, end, frameBuffer, vertices, topology, shader, uniforms, varyings, width) {
    var l = vertices.length; // Vertex index is required to associate varyings with fragments
    var v = 0;
    while (v < l) {
        var shape = assemblShape(topology, v, vertices);
        // check 4 corners of block against each shape:
        // TODO: FIRST, check if the bounding box of the shape overlaps this block
        var corners = triangleOverlapsSquare(topLeft, type ? ASCII_BLOCK_SIZE : PIXEL_BLOCK_SIZE, shape[0], shape[1], shape[2]);
        if (corners == 4) { // are all four corners inside of the triangle?
            for (var y = topLeft[1]; y < bottom; y++) {
                for (var x = topLeft[0]; x < end; x++) {
                    var varying = interpolateVaryings([x, y], shape, topology, v, varyings);
                    var value = shader([x, y], uniforms, varying);
                    drawPixel(type, x, y, value, frameBuffer, width);
                }
            }
        }
        else if (corners > 0) { // some of them are contained.
            for (var y = topLeft[1]; y < bottom; y++) {
                for (var x = topLeft[0]; x < end; x++) {
                    if (pointIsWithinTriangle([x, y], shape[0], shape[1], shape[2])) {
                        var varying = interpolateVaryings([x, y], shape, topology, v, varyings);
                        var value = shader([x, y], uniforms, varying);
                        drawPixel(type, x, y, value, frameBuffer, width);
                    }
                }
            }
        }
        v += 3;
    }
}
// by this point, shapes and bounding boxes are identified and associated with fragment shaders
var rasterize = new object_1.Builtin("rasterize", [object_1.ARRAY_OBJ, object_1.INTEGER_OBJ, object_1.ARRAY_OBJ, object_1.ARRAY_OBJ, object_1.ARRAY_OBJ], function (_, _2, _3, frameBuffer, type, vertices, toplogies, fragmentShaders, uniforms, varyings, pixelHeight) {
    var cols, rows;
    if (type == 0) {
        var width = frameBuffer.length / pixelHeight;
        cols = Math.ceil(width / PIXEL_BLOCK_SIZE);
        rows = Math.ceil(pixelHeight / PIXEL_BLOCK_SIZE);
    }
    else {
        cols = Math.ceil(frameBuffer[0].length / ASCII_BLOCK_SIZE);
        rows = Math.ceil(frameBuffer.length / ASCII_BLOCK_SIZE);
    }
    for (var r = 0; r < rows; r++) {
        for (var c = 0; c < cols; c++) {
            // how many corners of this block are inside the triangle?
            var topLeft = [c * (type ? ASCII_BLOCK_SIZE : PIXEL_BLOCK_SIZE), r * (type ? ASCII_BLOCK_SIZE : PIXEL_BLOCK_SIZE)];
            var bottom = topLeft[1] + (type ? ASCII_BLOCK_SIZE : PIXEL_BLOCK_SIZE);
            var end = topLeft[0] + (type ? ASCII_BLOCK_SIZE : PIXEL_BLOCK_SIZE);
            rasterizeBlock(type, topLeft, bottom, end, frameBuffer, vertices, toplogies[0], fragmentShaders[0], uniforms, varyings);
        }
    }
    return frameBuffer; // crucial to return this. Otherwise, the reference could be lost across runtime environments
});
var shadeVertices = new object_1.Builtin("shadeVertices", [object_1.ARRAY_OBJ, object_1.ARRAY_OBJ, object_1.HASH_OBJ, object_1.FUNCTION_OBJ], function (_, _2, _3, vertexShader, vertices, uniforms, attributes) {
    if (attributes === void 0) { attributes = []; }
    var varyings = [];
    for (var v in vertices) {
        var varying = {};
        vertices[v] = vertexShader(vertices[v], attributes, uniforms, varying).slice(0, 2);
        varyings.push(varying);
    }
    return {
        varyings: varyings,
        vertices: vertices
    };
});
var blit = new object_1.Builtin("blit", [object_1.ARRAY_OBJ, object_1.INTEGER_OBJ, object_1.ARRAY_OBJ, object_1.ARRAY_OBJ, object_1.ARRAY_OBJ], function (_, _2, _3, frameBuffer, RASTER_MODE, width) {
    var out = "";
    if (0 == RASTER_MODE) {
        var w = width * 3, h = frameBuffer.length / w;
        for (var y = 0; y < h; y++) {
            for (var x = 0; x < w; x += 3) {
                var idx = y * width + x;
                var value = [frameBuffer[idx], frameBuffer[idx + 1], frameBuffer[idx + 2]];
                out += colorRenderer(value, "█");
            }
            out += "\n";
        }
    }
    else {
        for (var x in frameBuffer) {
            out += frameBuffer[x].join("") + "\n";
        }
    }
    return out;
});
exports.ImageObject = builtin_util_1.makeBuiltinClass("Image", [
    ["Image", new object_1.Builtin("Image", [], function (_, scope) {
            return scope;
        })],
    ["getDimensions", new object_1.Builtin("getDImensions", [], function (_, scope) {
            return scope;
        })],
    ["getBuffer", new object_1.Builtin("getBuffer", [], function (_, scope) {
            return scope;
        })],
]);
var SURFACE_TOPOLOGY = builtin_util_1.makeBuiltinEnum(["TRIANGLE_LIST"] //, "TRIANGLE_STRIP", "TRIANGLE_FAN", "QUAD"]
);
var BLEND_MODE = builtin_util_1.makeBuiltinEnum(["ADD", "SUBTRACT", "MULTIPLY", "NONE"]);
var RASTER_MODE = builtin_util_1.makeBuiltinEnum(["PIXEL", "ASCII", "ANSI_COLOR", "ANSI_SOLID_COLOR"]);
var ASCII = new object_1.StringObject(" .`,-_*x=/|FXH$#@");
var triangleListCube = [
    // back-left
    [-0.5, -0.5, -0.5, 0.0],
    [-0.5, 0.5, -0.5, 0.0],
    [0.5, -0.5, -0.5, 0.0],
    // back-right
    [0.5, 0.5, -0.5, 0.0],
    [0.5, -0.5, -0.5, 0.0],
    [-0.5, 0.5, -0.5, 0.0],
    // front-left
    [-0.5, -0.5, 0.5, 0.0],
    [-0.5, 0.5, 0.5, 0.0],
    [0.5, -0.5, 0.5, 0.0],
    // front-right
    [0.5, 0.5, 0.5, 0.0],
    [0.5, -0.5, 0.5, 0.0],
    [-0.5, 0.5, 0.5, 0.0],
    // left-front
    [-0.5, -0.5, 0.5, 0.0],
    [-0.5, 0.5, -0.5, 0.0],
    [-0.5, -0.5, -0.5, 0.0],
    // left-back
    [-0.5, 0.5, -0.5, 0.0],
    [-0.5, -0.5, 0.5, 0.0],
    [-0.5, 0.5, 0.5, 0.0],
    // right-front
    // [ 0.5,  0.5, -0.5, 0.0],
    // [ 0.5,  0.5,  0.5, 0.0],
    // [ 0.5, -0.5,  0.5, 0.0],
    // // right-back
    // [ 0.5,  0.5,  0.5, 0.0],
    // [ 0.5,  0.5, -0.5, 0.0],
    // [ 0.5, -0.5, -0.5, 0.0],
    // // top-left
    // [ -0.5,  0.5, -0.5, 0.0],
    // [ -0.5,  0.5,  0.5, 0.0],
    // [  0.5,  0.5, -0.5, 0.0],
    // // top right
    // [  0.5,  0.5, 0.5, 0.0],
    // [  0.5,  0.5, -0.5, 0.0],
    // [ -0.5,  0.5,  0.5, 0.0],   
    // bottom-left
    [-0.5, -0.5, -0.5, 0.0],
    [-0.5, -0.5, 0.5, 0.0],
    [0.5, -0.5, -0.5, 0.0],
    // bottom right
    [0.5, -0.5, 0.5, 0.0],
    [0.5, -0.5, -0.5, 0.0],
    [-0.5, -0.5, 0.5, 0.0],
];
var Shape = builtin_util_1.makeBuiltinInterface([
    ["CUBE", new object_1.ArrayObject([
            [-0.5, -0.5, -0.5, 0.0],
            [0.5, -0.5, -0.5, 0.0],
            [0.5, 0.5, -0.5, 0.0],
            [-0.5, 0.5, -0.5, 0.0],
            [-0.5, -0.5, 0.5, 0.0],
            [0.5, -0.5, 0.5, 0.0],
            [0.5, 0.5, 0.5, 0.0],
            [-0.5, 0.5, 0.5, 0.0]
        ].map(function (r) { return obj_util_1.numberArrayToFloatArray(r); })),
    ],
    ["TRIANGLE_CUBE", new object_1.ArrayObject(triangleListCube.map(function (r) { return obj_util_1.numberArrayToFloatArray(r); }))]
]);
var Shaders = builtin_util_1.makeBuiltinInterface([
    ["VERTEX", builtin_util_1.makeBuiltinInterface([
            ["PROJECTION", builtin_util_1.makeBuiltinInterface([
                // ["PERSPECTIVE", new Builtin("perspectiveProjection", [ARRAY_OBJ, FLOAT_OBJ],
                //     function(context: WorkerContext, scope: Hash, jsScope: AnyObject, vertex: number[], cameraDistance: number){
                //         return vertex;
                //     })
                // ],
                // ["ORTHOGONAL", new Builtin("orthogonalProjection", [ARRAY_OBJ, FLOAT_OBJ],
                //     function(context: WorkerContext, scope: Hash, jsScope: AnyObject, vertex: number[], cameraDistance: number){
                //         return vertex;
                //     })
                // ]
                ])],
        ])],
    ["FRAGMENT", builtin_util_1.makeBuiltinInterface([
        //    ["DEPTH_PASS", new Builtin("depthPass", [ARRAY_OBJ],  function(context: WorkerContext, scope: Hash, jsScope: AnyObject, vertex: number[], cameraDistance: number){
        //     return vertex;
        //    })]
        ])]
]);
exports.Graphics = builtin_util_1.makeBuiltinInterface([
    ["SURFACE_TOPOLOGY", SURFACE_TOPOLOGY],
    ["BLEND_MODE", BLEND_MODE],
    ["RASTER_MODE", RASTER_MODE],
    ["ASCII", ASCII],
    ["SHAPE", Shape],
    ["Shaders", Shaders],
    ["makeBuffer", builtin_makeBuffer],
    ["shadeVertices", shadeVertices],
    ["rasterize", rasterize],
    ["blit", blit]
]);
