"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var token = require("./token");
var Lexer = /** @class */ (function () {
    function Lexer() {
        this.position = 0;
        this.readPosition = 0;
        this.ch = "";
        this.lineNumber = 1;
    }
    Lexer.prototype.setText = function (input) {
        this.position = 0;
        this.readPosition = 0;
        this.lineNumber = 1;
        this.ch = "";
        this.input = input;
    };
    Lexer.prototype.NextToken = function () {
        var tok = {}, peek = "", num;
        this.skipWhitespace();
        switch (this.ch) {
            case '=':
                peek = this.peekChar();
                if (peek == '=') {
                    tok.Literal = this.ch + peek;
                    tok.Type = token.EQ;
                    this.readChar();
                }
                else {
                    tok = newToken(token.ASSIGN, this.ch);
                }
                break;
            case '&':
                peek = this.peekChar();
                if (peek == '&') {
                    this.readChar();
                }
                tok = newToken(token.AND, this.ch + '&');
                break;
            case '|':
                peek = this.peekChar();
                if (peek == '|') {
                    this.readChar();
                }
                tok = newToken(token.OR, this.ch + '|');
                break;
            case '.':
                tok = newToken(token.DOT, this.ch);
                break;
            case '+':
                tok = newToken(token.PLUS, this.ch);
                break;
            case ',':
                tok = newToken(token.COMMA, this.ch);
                break;
            case ';':
                tok = newToken(token.SEMICOLON, this.ch);
                break;
            case ':':
                tok = newToken(token.COLON, this.ch);
                break;
            case '(':
                tok = newToken(token.LPAREN, this.ch);
                break;
            case ')':
                tok = newToken(token.RPAREN, this.ch);
                break;
            case '{':
                tok = newToken(token.LBRACE, this.ch);
                break;
            case '}':
                tok = newToken(token.RBRACE, this.ch);
                break;
            case '[':
                tok = newToken(token.LBRACKET, this.ch);
                break;
            case ']':
                tok = newToken(token.RBRACKET, this.ch);
                break;
            case '!':
                peek = this.peekChar();
                if (peek == '=') {
                    tok.Literal = this.ch + peek;
                    tok.Type = token.NOT_EQ;
                    this.readChar();
                }
                else {
                    tok = newToken(token.BANG, this.ch);
                }
                break;
            case '-':
                if (this.peekChar() == '>') {
                    tok.Literal = this.ch + '>';
                    tok.Type = token.SOURCE;
                    this.readChar();
                }
                else {
                    tok = newToken(token.MINUS, this.ch);
                }
                break;
            case '/':
                if (this.peekChar() == '*') {
                    tok = newToken(token.LCOMMENT, this.ch + '*');
                    this.readChar();
                    break;
                }
                else {
                    tok = newToken(token.SLASH, this.ch);
                    break;
                }
            case '*':
                if (this.peekChar() == '/') {
                    tok = newToken(token.RCOMMENT, this.ch + '/');
                    this.readChar();
                }
                else {
                    tok = newToken(token.ASTERISK, this.ch);
                }
                break;
            case '%':
                tok = newToken(token.MOD, this.ch);
                break;
            case '<':
                if (this.peekChar() == '-') {
                    tok.Literal = this.ch + '-';
                    tok.Type = token.SINK;
                    this.readChar();
                }
                else {
                    tok = newToken(token.LT, this.ch);
                    break;
                }
            case '>':
                tok = newToken(token.GT, this.ch);
                break;
            case '"':
            case "'":
                tok.Type = token.STRING;
                tok.Literal = this.readString(this.ch);
                break;
            case 'Ω':
                tok.Literal = "Ω";
                tok.Type = token.EOF;
                break;
            default:
                if (isLetter(this.ch)) {
                    tok.Literal = this.readIdentifier();
                    tok.Type = token.LookupIdent(tok.Literal);
                    return tok;
                }
                else if (isDigit(this.ch)) {
                    num = this.readNumber();
                    tok.Literal = num[1];
                    tok.Type = num[0] ? token.FLOAT : token.INT;
                    return tok;
                }
                else {
                    tok = newToken(token.ILLEGAL, this.ch);
                }
        }
        this.readChar();
        return tok;
    };
    Lexer.prototype.getInputLength = function () {
        return this.input.length;
    };
    Lexer.prototype.skipWhitespace = function () {
        while (this.ch == ' ' || this.ch == '\t' || this.ch == '\r' || this.isNewLine()) {
            this.readChar();
        }
    };
    Lexer.prototype.peekChar = function () {
        if (this.readPosition >= this.input.length) {
            return 'Ω';
        }
        else {
            return this.input[this.readPosition];
        }
    };
    Lexer.prototype.readChar = function () {
        this.ch = this.peekChar();
        this.position = this.readPosition;
        this.readPosition += 1;
    };
    Lexer.prototype.readIdentifier = function () {
        var position = this.position;
        if (isLetter(this.ch)) {
            this.readChar();
        }
        while (isLetter(this.ch) || isWholeDigit(this.ch)) {
            this.readChar();
        }
        return this.input.substring(position, this.position);
    };
    Lexer.prototype.readNumber = function () {
        var position = this.position, float = false;
        while (isDigit(this.ch)) {
            if (this.ch == ".") {
                float = true;
            }
            this.readChar();
        }
        return [float, this.input.substring(position, this.position)];
    };
    Lexer.prototype.readString = function (endChar) {
        var position = this.position + 1;
        while (true) {
            this.readChar();
            if (this.ch == endChar) {
                break;
            }
        }
        return this.input.substring(position, this.position);
    };
    Lexer.prototype.isNewLine = function () {
        var newLine = this.ch == '\n';
        if (newLine) {
            this.lineNumber++;
        }
        return newLine;
    };
    return Lexer;
}());
exports.Lexer = Lexer;
function isWholeDigit(ch) {
    return ('0' <= ch && ch <= '9');
}
exports.isWholeDigit = isWholeDigit;
function isDigit(ch) {
    return ('0' <= ch && ch <= '9') || ch == '.';
}
exports.isDigit = isDigit;
function isLetter(ch) {
    return 'a' <= ch && ch <= 'z' || 'A' <= ch && ch <= 'Z' || ch == '_' || ch && ch.charCodeAt(0) > 200;
}
exports.isLetter = isLetter;
function newToken(tokenType, ch) {
    return { Type: tokenType, Literal: ch };
}
exports.newToken = newToken;
