import { Environment } from './environment';
import * as statement from '../ast/statement';
import * as literal from '../ast/literal';
import { STREAM_DIRECTION } from '../token';
import { AnyObject } from '../util';
import { WorkerContext } from '../runtime/events';
import { Optimizer } from '../runtime/optimizer';
export declare type ObjectType = string;
export declare type inspectTypes = string | number | boolean | null;
export interface EObject {
    Type: () => ObjectType;
    Inspect: (indentLevel?: number) => inspectTypes;
}
declare const INTEGER_OBJ = "int", FLOAT_OBJ = "float", BOOLEAN_OBJ = "boolean", NULL_OBJ = "null", RETURN_VALUE_OBJ = "RETURN_VALUE", ERROR_OBJ = "error", FUNCTION_OBJ = "function", PURE_FUNCTION_OBJ = "pure_function", STRING_OBJ = "string", BUILTIN_OBJ = "BUILTIN", ARRAY_OBJ = "array", HASH_OBJ = "hash";
export declare type BuiltinFunction = (context?: WorkerContext, scope?: AnyObject, jsScope?: AnyObject, ...args: any[]) => any;
export declare class Null implements EObject {
    constructor();
    Value: any;
    Type(): ObjectType;
    Inspect(): inspectTypes;
}
export declare class Integer implements EObject {
    Value: number;
    constructor(Value: number);
    Inspect(): inspectTypes;
    Type(): ObjectType;
    HashKey(): string;
}
export declare class Float implements EObject {
    Value: number;
    constructor(Value: number);
    Inspect(): inspectTypes;
    Type(): ObjectType;
    HashKey(): string;
}
export declare class BooleanObject implements EObject {
    Value: boolean;
    constructor(Value: boolean);
    Type(): ObjectType;
    Inspect(): inspectTypes;
    HashKey(): string;
}
export declare class ReturnValue implements EObject {
    Value: EObject;
    constructor(Value: EObject);
    Type(): ObjectType;
    Inspect(): inspectTypes;
}
export interface FunctionObject extends EObject {
    Parameters?: literal.Identifier[];
    Fn?: BuiltinFunction;
    ReturnType?: string;
    ParameterTypes?: string[];
}
export interface DynamicFunction extends FunctionObject {
    Parameters: literal.Identifier[];
    Body: statement.BlockStatement;
    Env?: Environment;
    ReturnType?: string;
    ParameterTypes?: string[];
    stateful?: string[];
}
export declare class LambdaFunction implements DynamicFunction {
    Parameters: literal.Identifier[];
    Body: statement.BlockStatement;
    Env: Environment;
    Fn?: BuiltinFunction;
    ObjectContext?: Hash;
    ReturnType?: string;
    ParameterTypes?: string[];
    stateful: any;
    constructor(Parameters: literal.Identifier[], Body: statement.BlockStatement, Env: Environment, Fn?: BuiltinFunction, ObjectContext?: Hash, ReturnType?: string, ParameterTypes?: string[], stateful?: any);
    Type(): ObjectType;
    Inspect(indentLevel?: number): inspectTypes;
}
export declare class PureFunction implements DynamicFunction {
    Parameters: literal.Identifier[];
    Body: statement.BlockStatement;
    Env?: Environment;
    Fn?: BuiltinFunction;
    ReturnType?: string;
    ParameterTypes?: string[];
    constructor(Parameters: literal.Identifier[], Body: statement.BlockStatement, Env?: Environment, Fn?: BuiltinFunction, ReturnType?: string, ParameterTypes?: string[]);
    Type(): ObjectType;
    Inspect(indentLevel?: number): inspectTypes;
}
export declare class Builtin implements FunctionObject {
    name: string;
    signature: [] | (ObjectType | ObjectType[])[];
    Fn: BuiltinFunction;
    Context?: string;
    ObjectContext?: Hash;
    stateful?: string[];
    ecsOnly: boolean;
    builtin: boolean;
    constructor(name: string, signature: [] | (ObjectType | ObjectType[])[], Fn: BuiltinFunction, Context?: string, ObjectContext?: Hash, stateful?: string[], ecsOnly?: boolean);
    callFromECSRuntime(context: WorkerContext, scope: Hash, optimizer: Optimizer, params: EObject[]): EObject;
    Type(): ObjectType;
    Inspect(indentLevel?: number): inspectTypes;
    private decorateSignature;
}
export declare class Stream implements FunctionObject {
    Direction: STREAM_DIRECTION;
    Transforms: FunctionObject[];
    Source?: FunctionObject;
    Sink?: FunctionObject;
    ReturnType?: string;
    constructor(Direction: STREAM_DIRECTION, Transforms?: FunctionObject[], Source?: FunctionObject, Sink?: FunctionObject, ReturnType?: string);
    Type(): ObjectType;
    Inspect(indentLevel?: number): inspectTypes;
}
export declare class StringObject implements EObject {
    Value: string;
    constructor(Value: string);
    Type(): ObjectType;
    Inspect(): inspectTypes;
    HashKey(): string;
}
export declare class ArrayObject implements EObject {
    Elements: EObject[];
    constructor(Elements: EObject[]);
    Type(): ObjectType;
    Inspect(): inspectTypes;
}
export declare class ErrorObject implements EObject {
    Message: string;
    constructor(Message: string);
    Value: string;
    Type(): ObjectType;
    Inspect(): inspectTypes;
}
export declare type HashKey = {
    Type: ObjectType;
    Value: any;
};
export declare type HashPair = {
    Key: EObject;
    Value: EObject;
    modifiers?: number[];
};
export interface Hashable {
    HashKey(): HashKey;
}
export declare class Hash implements EObject {
    Pairs: {
        [key: string]: HashPair;
    };
    Constructor?: FunctionObject;
    className?: string;
    builtins: AnyObject;
    constructor(Pairs: {
        [key: string]: HashPair;
    }, Constructor?: FunctionObject, className?: string, builtins?: AnyObject);
    Type(): ObjectType;
    Inspect(indentLevel?: number): inspectTypes;
}
export { INTEGER_OBJ, FLOAT_OBJ, BOOLEAN_OBJ, NULL_OBJ, RETURN_VALUE_OBJ, ERROR_OBJ, FUNCTION_OBJ, PURE_FUNCTION_OBJ, STRING_OBJ, BUILTIN_OBJ, ARRAY_OBJ, HASH_OBJ };
export declare const strBuiltin: StringObject;
export declare const TRUE: BooleanObject;
export declare const FALSE: BooleanObject;
export declare const NULL: Null;
