"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("../ast/util");
/**
 * Analyzer creates a ParseTreeAnalysis from
 * a pre-parsed Program AST node, much like the parser would.
 *
 * The use-case is that a program may already be parsed either from:
 * - compiling hot functions,
 * - or importing a Program from an AST editor.
 */
var Analyzer = /** @class */ (function () {
    function Analyzer() {
    }
    Analyzer.prototype.analyzeParseTree = function (program) {
        return this.analyzeNode(program, this.initAnalysis());
    };
    Analyzer.prototype.analyzeNode = function (node, analysis) {
        var isLoop = this.isLoop(node);
        analysis.hasLoops = analysis.hasLoops || isLoop;
        switch (node.NodeName) {
            case "Program":
            case "BlockStatement":
                this.analyzeBlockStatement(node, analysis);
                break;
            case "ExpressionStatement":
                this.analyzeNode(node.Expression, analysis);
                break;
            case "InfixExpression":
                this.analyzeNode(node.Left, analysis);
                this.analyzeNode(node.Right, analysis);
                break;
            case "PrefixExpression":
                this.analyzeNode(node.Right, analysis);
                break;
            case "CallExpression":
                this.analyzeCallExpression(node, analysis);
            case "Identifier":
                this.analyzeExternalVars(node, analysis);
                break;
            case "LetStatement":
                this.analyzeLetStatement(node, analysis);
                break;
            case "AssignmentStatement":
                this.analyzeNode(node.Name, analysis);
                this.analyzeNode(node.Value, analysis);
                break;
            case "Function":
            case "FunctionLiteral":
                this.analyzeFunction(node, analysis);
                break;
            case "ForStatement":
                analysis.declaredVariables[node.Element.Value] =
                    util_1.getDataTypeByNodeName(node.Range) === "hash"
                        ? "string"
                        : "int";
                this.analyzeNode(node.Range, analysis);
                this.analyzeNode(node.Consequence, analysis);
                break;
            case "WhileStatement":
                this.analyzeNode(node.Condition, analysis);
                this.analyzeNode(node.Consequence, analysis);
                break;
        }
        return analysis;
    };
    Analyzer.prototype.isLoop = function (node) {
        if (node.NodeName === "ForStatement" || node.NodeName === "WhileStatement") {
            return true;
        }
        return false;
    };
    Analyzer.prototype.analyzeCallExpression = function (node, analysis) {
        for (var x = 0; x < node.Arguments.length; x++) {
            this.analyzeNode(node.Arguments[x], analysis);
        }
        if (node.Function.NodeName == "FunctionLiteral") {
            this.analyzeBlockStatement(node.Function.Body, analysis);
        }
        else {
            var funcIdent = node.Function.Value;
            if (!analysis.declaredVariables[funcIdent]) {
                analysis.undeclaredVariables[funcIdent] = true;
            }
        }
    };
    Analyzer.prototype.analyzeFunction = function (node, analysis) {
        var signature = node.ParameterTypes;
        for (var p = 0, params = node.Parameters, l = params.length; p < l; p++) {
            analysis.declaredVariables[params[p].Value] = signature[p] || "any";
        }
        this.analyzeBlockStatement(node.Body, analysis);
    };
    Analyzer.prototype.analyzeBlockStatement = function (node, analysis) {
        for (var s in node.Statements) {
            this.analyzeNode(node.Statements[s], analysis);
        }
    };
    Analyzer.prototype.analyzeLetStatement = function (node, analysis) {
        if (node.Vars) {
            var vars = node.Vars;
            for (var v in vars) {
                analysis.declaredVariables[vars[v][0].Value] = (node.Types[v])
                    || util_1.getDataTypeByNodeName(vars[v][1]);
            }
        }
        else {
            analysis.declaredVariables[node.Name.Value] = node.dataType
                || util_1.getDataTypeByNodeName(node.Value);
        }
    };
    Analyzer.prototype.analyzeExternalVars = function (node, analysis) {
        if (!analysis.declaredVariables[node.Value]) {
            analysis.undeclaredVariables[node.Value] = true;
        }
    };
    Analyzer.prototype.initAnalysis = function () {
        return {
            declaredVariables: {},
            undeclaredVariables: {},
            hasLoops: false,
            hasRecursion: false
        };
    };
    return Analyzer;
}());
exports.Analyzer = Analyzer;
