"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var expression_1 = require("../ast/expression");
var literal_1 = require("../ast/literal");
var token_1 = require("../token");
var Transpiler = /** @class */ (function () {
    function Transpiler() {
        var _this = this;
        this.config = {};
        // like the procedure divison, except for external identifiers
        this.injectedHeader = "";
        this.injectedContextUpdate = "";
        this.transpile = function (node, env, analysis, plugin, topScope // true when parent node is Program
        ) {
            if (node.NodeName === "Program") {
                _this.env = env;
                _this.analysis = analysis;
                _this.injectedContextUpdate = "";
                _this.injectedHeader = "";
                if (plugin) {
                    plugin.setTranspiler(_this);
                    _this.plugin = plugin;
                }
                else {
                    _this.onRunWithoutPlugins();
                }
            }
            if (!node) {
                console.log("Warning: Node is undefined");
                return "";
            }
            if (_this.plugin && !_this.config.noPlugins) {
                var transformPlugin = _this.plugin.nodeTransforms[node.NodeName];
                if (transformPlugin) {
                    var nodeAndHeader = transformPlugin(node, topScope);
                    node = nodeAndHeader[0];
                    _this.injectedHeader += nodeAndHeader[1];
                    _this.injectedContextUpdate += (nodeAndHeader[2] ? nodeAndHeader[2] : "");
                }
            }
            if (node.NodeName === "Identifier") {
                node.Value = _this.replaceBuiltinIdentifiers(node.Value);
            }
            if (node.NodeName === "CallExpression") {
                node = _this.transpileCallExpression(node);
            }
            var nodeTransform = _this.delta[node.NodeName];
            if (nodeTransform) {
                return nodeTransform(node, _this);
            }
            return node.UnParse(_this);
        };
        this.transpileBlock = function (statements, topScope) {
            var result = "";
            for (var s = 0, l = statements.length; s < l; s++) {
                var stmt = statements[s];
                result += _this.transpile(stmt, null, null, null, topScope) + _this.endOfStatementDelimiter;
            }
            return result;
        };
        this.callIndex = new literal_1.StringLiteral({ Literal: "call", Type: token_1.STRING }, "call");
        this.callBuiltinIndex = new literal_1.StringLiteral({ Literal: "callBuiltin", Type: token_1.STRING }, "callBuiltin");
        this.callMethodIndex = new literal_1.StringLiteral({ Literal: "callBuiltinMethod", Type: token_1.STRING }, "callMethod");
        this.getBuiltinObjectIndex = new literal_1.StringLiteral({ Literal: "getBuiltinObject", Type: token_1.STRING }, "getBuiltinObject");
        this.getterIndex = new literal_1.StringLiteral({ Literal: "getVar", Type: token_1.STRING }, "getVar");
        this.setterIndex = new literal_1.StringLiteral({ Literal: "setVar", Type: token_1.STRING }, "setVar");
        this.indexToken = { Literal: "[", Type: "[" };
        this.left = new literal_1.Identifier({ Literal: "ctx", Type: token_1.IDENT }, "ctx");
    }
    Transpiler.prototype.unParse = function (node, config) {
        if (config && config.noPlugins) {
            var out = void 0;
            this.config.noPlugins = true;
            out = node.UnParse(this);
            this.config.noPlugins = false;
            return out;
        }
        return node.UnParse(this);
    };
    Transpiler.prototype.getProgramHeader = function () {
        return this.injectedHeader ? this.injectedHeader : "";
    };
    Transpiler.prototype.transpileCallExpression = function (node) {
        var callFunction = node.Function
            ? (node.Function)
            : null;
        // console.log("transpiled call expression ", node)
        if (!callFunction) {
            return node;
        }
        var methodName;
        if (callFunction.NodeName == "Identifier") {
            methodName = callFunction.Value;
        }
        if (callFunction.NodeName === "IndexExpression") {
            if (callFunction.Left.NodeName === "Identifier") {
                var polyfillAPI = this.methodCallPolyfill[callFunction.Left.Value];
                if (polyfillAPI) {
                    var polyfillMethodTransformer = polyfillAPI[callFunction.Index.Value];
                    if (polyfillMethodTransformer) {
                        node = polyfillMethodTransformer(node);
                    }
                }
            }
        }
        if (methodName) {
            if (this.callExpressionPolyfill[methodName]) {
                node = this.callExpressionPolyfill[methodName](node);
            }
            else {
                node.Function.Value
                    = this.replaceBuiltinIdentifiers(node.Function.Value);
            }
        }
        return node;
    };
    Transpiler.prototype.getBuiltinFunctionList = function () {
        return Object.keys(this.builtinFunctionPolyfill).concat(Object.keys(this.callExpressionPolyfill));
    };
    Transpiler.prototype.routeThroughContext = function (indexIdent, args) {
        var routed = new expression_1.CallExpression({}, new expression_1.IndexExpression(this.indexToken, this.left, new literal_1.StringLiteral({ Type: '"', Literal: indexIdent.Value }, indexIdent.Value)), args);
        return routed;
    };
    Transpiler.prototype.replaceBuiltinIdentifiers = function (identName) {
        var polyfill = this.builtinFunctionPolyfill[identName];
        if (polyfill) {
            return polyfill;
        }
        return identName;
    };
    Transpiler.prototype.getTypeFromIdentifierName = function (identName) {
        var declaredInProgram = this.analysis.declaredVariables[identName];
        if (declaredInProgram) {
            return declaredInProgram;
        }
        else {
            var preExistingIdentValue = this.env.get(identName);
            if (preExistingIdentValue) {
                return preExistingIdentValue.Type();
            }
        }
        return "any";
    };
    Transpiler.prototype.inferTypeFromLiteral = function (node) {
        var theType = this.literalToTypeMapping[node.NodeName];
        if (theType) {
            return theType;
        }
    };
    Transpiler.prototype.getTypeFromECSType = function (typeName, defaultType) {
        if (defaultType === void 0) { defaultType = "any"; }
        if (this.typeToTypeMapping[typeName]) {
            typeName = this.typeToTypeMapping[typeName];
        }
        return typeName || defaultType;
    };
    Transpiler.prototype.firstArgumentToObjectTransform = function (node, callName) {
        var indexExpr = new expression_1.IndexExpression({ Literal: "[", Type: "[" }, node.Arguments[0], new literal_1.StringLiteral({ Literal: callName, Type: "IDENT" }, callName));
        var expr = new expression_1.CallExpression({ Literal: "(", Type: "(" }, indexExpr, [node.Arguments[1]]);
        return expr;
    };
    return Transpiler;
}());
exports.Transpiler = Transpiler;
